
/*
 * tangle-misc.h
 *
 * This file is part of Tangle Toolkit - A graphical widget library based on Clutter Toolkit
 *
 * (c) 2009-2010 Henrik Hedberg <henrik.hedberg@innologies.fi>
 *
 */

#ifndef __TANGLE_MISC_H__
#define __TANGLE_MISC_H__

#include <glib-object.h>
#include <clutter/clutter.h>

typedef struct _TangleValueCalculationFunctions {
	gint (*compare)(const GValue* value_a, const GValue* value_b);
	void (*add)(GValue* value, const GValue* addend);
	void (*substract)(GValue* value, const GValue* subtrahend);
	void (*multiply)(GValue* value, const GValue* multiplier);
	void (*divide)(GValue* value, const GValue* divider);
} TangleValueCalculationFunctions;

gboolean tangle_init(int* argc, char*** argv);

#define TANGLE_UNREF_AND_NULLIFY_OBJECT(x) ((x) = tangle_unref_object(x))
#define TANGLE_UNPARENT_AND_NULLIFY_ACTOR(x) ((x) = tangle_unparent_actor(x))

gpointer tangle_unref_object(gpointer object);
ClutterActor* tangle_unparent_actor(ClutterActor* actor);

typedef gpointer TanglePointers;

TanglePointers* tangle_pointers_new(gint n, ...);
void tangle_pointers_get(TanglePointers* pointers, gint n, ...);
void tangle_pointers_free(TanglePointers* pointers, gint n);

#define TANGLE_POINTERS(p) ((TanglePointers*)(p))

gboolean tangle_lookup_construct_properties(GType type, guint n_construct_properties, GObjectConstructParam* construct_properties, const gchar* first_property_name, ...);
GObject* tangle_construct_with_extra_properties(GObject* (*constructor)(GType, guint, GObjectConstructParam*), GType type, guint n_construct_properties, GObjectConstructParam* construct_properties, const gchar* first_extra_property_name, ...);

gboolean tangle_signal_accumulator_non_null_handled(GSignalInvocationHint* ihint, GValue* return_accu, const GValue* handler_return, gpointer user_data);

gboolean tangle_signal_connect_dynamically(GObject* object, const gchar* signal_name, const gchar* function_name, gpointer user_data);
gboolean tangle_signal_connect_swapped_dynamically(GObject* object, const gchar* signal_name, const gchar* function_name, gpointer user_data);
gboolean tangle_signal_connect_object_swapped_dynamically(GObject* object, const gchar* signal_name, const gchar* function_name, GObject* other_object);

gboolean tangle_signal_connect_dynamically_from_script(GObject* object, const gchar* signal_name, const gchar* function_name, ClutterScript* script);

gint tangle_value_compare(const GValue* value, const GValue* comparable);
void tangle_value_add(GValue* value, const GValue* addend);
void tangle_value_substract(GValue* value, const GValue* subtrahend);
void tangle_value_multiply(GValue* value, const GValue* multiplier);
void tangle_value_divide(GValue* value, const GValue* divider);

gboolean tangle_value_is_calculation_supported(GType value_type);
void tangle_value_register_calculation_functions(GType value_type, TangleValueCalculationFunctions* calculation_functions);

gchar* tangle_lookup_filename(const gchar* filename);
void tangle_add_search_path(const gchar* search_path);

ClutterAnimation* tangle_object_animate(GObject* object, gulong mode, guint duration, ...);
ClutterAnimation* tangle_object_animate_with_timeline(GObject* object, gulong mode, ClutterTimeline* timeline, ...);
ClutterAnimation* tangle_object_animatev(GObject* object, gulong mode, guint duration, gint n_properties, const gchar* const property_names[], const GValue* property_values);
ClutterAnimation* tangle_object_animate_valist(GObject* object, gulong mode, guint duration, va_list properties);
ClutterAnimation* tangle_object_animate_valist_with_timeline(GObject* object, gulong mode, ClutterTimeline* timeline, va_list properties);
gboolean tangle_object_stop_animation(GObject* object, ClutterAnimation* animation);

#endif
