/*
 * tangle-widget.h
 *
 * This file is part of Tangle Toolkit - A graphical widget library based on Clutter Toolkit
 *
 * (c) 2010 Henrik Hedberg <henrik.hedberg@innologies.fi>
 *
 */

#ifndef __TANGLE_WIDGET_H__
#define __TANGLE_WIDGET_H__

#include "tangle-actor.h"

#define TANGLE_TYPE_WIDGET (tangle_widget_get_type())
#define TANGLE_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), TANGLE_TYPE_WIDGET, TangleWidget))
#define TANGLE_IS_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), TANGLE_TYPE_WIDGET))
#define TANGLE_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), TANGLE_TYPE_WIDGET, TangleWidgetClass))
#define TANGLE_IS_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), TANGLE_TYPE_WIDGET))
#define TANGLE_WIDGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), TANGLE_TYPE_WIDGET, TangleWidgetClass))

typedef struct _TangleWidgetPrivate TangleWidgetPrivate;
typedef struct _TangleWidget TangleWidget;
typedef struct _TangleWidgetClass TangleWidgetClass;

struct _TangleWidget {
	TangleActor parent_instance;
	TangleWidgetPrivate* priv;
};

struct _TangleWidgetClass {
	TangleActorClass parent_class;
	
	void (*allocate_children)(TangleWidget* widget, const ClutterActorBox* box, ClutterAllocationFlags flags);
	void (*paint_children)(TangleWidget* widget);
};

typedef void (*TangleSwappedCallback)(gpointer data, ClutterActor* actor);
#define TANGLE_SWAPPED_CALLBACK(f) ((TangleSwappedCallback)(f))

GType tangle_widget_get_type(void) G_GNUC_CONST;

#include "tangle-style.h"
#include "tangle-layout.h"
#include "tangle-clamp.h"

ClutterActor* tangle_widget_new(void);
ClutterActor* tangle_widget_new_with_layout(TangleLayout* layout);
ClutterActor* tangle_widget_new_scrollable(TangleLayout* layout);

guint tangle_widget_get_n_children(TangleWidget* widget);
ClutterActor* tangle_widget_get_nth_child(TangleWidget* widget, guint n);
gboolean tangle_widget_has_child(TangleWidget* widget, ClutterActor* actor);
GList* tangle_widget_get_children_readonly(TangleWidget* widget);
void tangle_widget_foreach_swapped(TangleWidget* widget, TangleSwappedCallback swapped_callback, gpointer user_data);
void tangle_widget_foreach_type(TangleWidget* widget, GType type, ClutterCallback callback, gpointer user_data);
void tangle_widget_foreach_type(TangleWidget* widget, GType type, ClutterCallback callback, gpointer user_data);
void tangle_widget_foreach_type_swapped(TangleWidget* widget, GType type, TangleSwappedCallback swapped_callback, gpointer user_data);

void tangle_widget_add(TangleWidget* widget, ClutterActor* actor, ...) G_GNUC_NULL_TERMINATED;
void tangle_widget_add_before(TangleWidget* widget, ClutterActor* actor, ClutterActor* existing, ...) G_GNUC_NULL_TERMINATED;
void tangle_widget_add_after(TangleWidget* widget, ClutterActor* actor, ClutterActor* existing, ...) G_GNUC_NULL_TERMINATED;

void tangle_widget_set_layout(TangleWidget* widget, TangleLayout* layout);
TangleLayout* tangle_widget_get_layout(TangleWidget* widget);

void tangle_widget_get_padding(TangleWidget* widget, TangleSpacing* spacing);
void tangle_widget_set_padding(TangleWidget* widget, const TangleSpacing* spacing);
gboolean tangle_widget_get_padding_set(TangleWidget* widget);
void tangle_widget_set_padding_set(TangleWidget* widget, gboolean is_set);

void tangle_widget_get_background_color(TangleWidget* widget, ClutterColor* color);
gboolean tangle_widget_get_background_color_set(TangleWidget* widget);
void tangle_widget_set_background_color(TangleWidget* widget, const ClutterColor* color);

ClutterActor* tangle_widget_get_background_actor(TangleWidget* widget);
void tangle_widget_set_background_actor(TangleWidget* widget, ClutterActor* actor);
void tangle_widget_change_background_actor(TangleWidget* widget, ClutterActor* actor);

ClutterActor* tangle_widget_get_foreground_actor(TangleWidget* widget);
void tangle_widget_set_foreground_actor(TangleWidget* widget, ClutterActor* actor);

gboolean tangle_widget_get_prefer_background_size(TangleWidget* widget);
void tangle_widget_set_prefer_background_size(TangleWidget* widget, gboolean prefer_background_size);
gboolean tangle_widget_get_prefer_background_size_set(TangleWidget* widget);
void tangle_widget_set_prefer_background_size_set(TangleWidget* widget, gboolean is_set);

void tangle_widget_get_real_preferred_size(TangleWidget* widget, gboolean interacting, gfloat* min_width_p, gfloat* min_height_p, gfloat* natural_width_p, gfloat* natural_height_p, gfloat* max_width_p, gfloat* max_height_p);
void tangle_widget_get_real_preferred_width(TangleWidget* widget, gfloat for_height, gboolean interacting, gfloat* min_width_p, gfloat* natural_width_p, gfloat* max_width_p);
void tangle_widget_get_real_preferred_height(TangleWidget* widget, gfloat for_width, gboolean interacting, gfloat* min_height_p, gfloat* natural_height_p, gfloat* max_height_p);

gboolean tangle_widget_get_descendant_interacting(TangleWidget* widget);
gboolean tangle_widget_get_descendant_dragging(TangleWidget* widget);

void tangle_widget_clamp_child_boundaries(TangleWidget* widget, TangleClamp* clamp);

#endif
